/* A colletion of functions to plot quakeML data */

var Debug = false;


/* Plot a google map, similarly to ewhtmlreport */
function QMLGMap( opt )
{
	var self = this;


	/* Check options */
	if( typeof opt.placeholderID === "undefined" )
	{
		console.error( "Invalid options." )
		return null;
	}

	/* Default options */
	self.gmapopt =
			{
				zoom: 3,
				center: new google.maps.LatLng( 0, 0 ),
				mapTypeId: google.maps.MapTypeId.HYBRID,
				streetViewControl: false
			};
	self.iconopt = {
		scaleopt: [1, 2],
		path: google.maps.SymbolPath.CIRCLE,
		fillOpacity: 0.5,
		strokeOpacity: 0.5,
		strokeColor: "#FF0000",
		fillColor: "#FF0000",
		strokeWeight: 1
	};


	/* Merge options with defaults */
	$.extend( true, self, opt );

	/* Create shared infowindow */
	self.infoWindow = new google.maps.InfoWindow()

	/* Start google map */
	self.gmap = new google.maps.Map(
			document.getElementById( self.placeholderID ),
			self.gmapopt );

	/* Update google map with first events */
	self.markers = [];
	if( typeof opt.xmldata !== "undefined" &&
			opt.xmldata !== null &&
			$( opt.xmldata ).length > 0 )
		self.update( opt.xmldata, true );
}



QMLGMap.prototype.update = function( xmldata, updateBounds, changedIcon )
{
	var self = this;
	
	/* Add new markers */
	var finalmarkers = [];
	$( xmldata ).find( "event" ).each( function( index, newevent ) {
		/* Extract marker parameters */
		var eventId = $( newevent ).attr( "publicID" );
		/* Preferred Origin */
		var preferredOriginID = $( this ).find( "preferredOriginID" ).text();
		if( Debug )
			console.log( "Pref. Origin: " + preferredOriginID );
		var porigin = $( newevent ).find( "[publicID|=" +
				self.jQuerySelectorEscape( preferredOriginID ) + "]" );
		if( porigin.length === 0 )
		{
			if( Debug )
				console.log( "Did not find preferred origin." );
			return true;
		}
		if( Debug )
			console.log( "Found preferred origin." );

		/* Location */
		var ot = $( porigin ).find( "time" ).find( "value" ).text();
		var lat = parseFloat( $( porigin ).find( "latitude" ).text() );
		var lon = parseFloat( $( porigin ).find( "longitude" ).text() );
		var z = parseFloat( $( porigin ).find( "depth" ).text() ) / 1000;
		var location = new google.maps.LatLng( lat, lon );


		/* Preferred Magnitude */
		var preferredMagnitudeID = $( newevent ).find( "preferredMagnitudeID" ).text();
		if( Debug )
			console.log( "Pref. Mag: " + preferredMagnitudeID );
		var pmag = $( newevent ).find( "[publicID|=" +
				self.jQuerySelectorEscape( preferredMagnitudeID ) + "]" );

		/* Marker scale */
		var scale = self.iconopt.scaleopt[0];
		var mag = "";
		var magtype = "";
		if( $( pmag ).length > 0 )
		{
			if( Debug )
				console.log( "Found preferred magnitude." );
			mag = parseFloat( $( pmag ).find( "mag" ).find( "value" ).text() );
			magtype = $( pmag ).find( "type" ).text();
			scale = self.getMarkerScale( mag );
		}
		else
		{
			if( Debug )
				console.log( "Did not find preferred magnitude." );
		}

		/* Infowindow contents */
		var infoContent = '<table style="color:#8888aa;font-family: sans-serif;font-size: 12px;">' +
				'<tr><th>EW Event ID: ' + eventId + '</th></tr>' +
				'<tr style="background-color:#eeeeff"><td>Origin Time</td><td>' + ot + '</td></tr>' +
				'<tr><td>Latitude</td><td>' + lat + '</td></tr>' +
				'<tr style="background-color:#eeeeff"><td>Longitude</td><td>' + lon + '</td></tr>' +
				'<tr><td>Depth</td><td>' + z + '</td></tr>' +
				'<tr style="background-color:#eeeeff"><td>Magnitude</td><td>' + mag + " " + magtype + '</td></tr>' + '</table>'

		/* Check if this is an old marker */
		var oldmarker = $.grep( self.markers, function( marker, index ) {
			if( marker.Id === eventId )
				return true;
			return false;
		} );

		var marker;
		if( oldmarker.length > 0 )
		{
			/* This is a marker to be updated */
			/* Location */
			oldmarker[0].setPosition( location );
			/* Icon - Only updates magnitude */
			var oldicon = oldmarker[0].getIcon();
			$.extend( true, oldicon, { scale: scale } );
			oldmarker[0].setIcon( oldicon );
		}
		else
		{
			/* This is a new marker */
			/* Create google marker */
			var newicon = { scale: scale };
			if( changedIcon )
				$.extend( newicon, changedIcon );
			else
				$.extend( newicon, self.iconopt );
			marker = new google.maps.Marker(
					{
						position: location,
						map: self.gmap,
						flat: true,
						icon: newicon,
						zIndex: 1000,
						/* Custom parameters */
						Id: eventId,
						InfoContent: infoContent
					} );
			/* Set event for infobox */
			google.maps.event.addListener( marker, 'click', function()
			{
				//var event = events[marker.eventindex];
				self.infoWindow.setContent( marker.InfoContent );
				self.infoWindow.open( self.gmap, marker );
			} );
		}
		/* Add marker to array of markers */
		finalmarkers.push( marker );
	} );

	/* Replace old marker array */
	self.markers = finalmarkers;

	/* Check marker bounds */
	if( updateBounds )
	{
		var bounds = new google.maps.LatLngBounds();
		$( self.markers ).each( function( index, marker ) {
			bounds.extend( marker.getPosition() );
		} );
		self.gmap.fitBounds( bounds );
		return bounds;
	}
};

/* Determine the scale of a marker from the event magnitude */
QMLGMap.prototype.getMarkerScale = function( mag )
{
	var self = this;
	return self.iconopt.scaleopt[0] + mag * self.iconopt.scaleopt[1];
};

/* Automatically escape a jquery selector */
QMLGMap.prototype.jQuerySelectorEscape = function( expression ) {
	return expression.replace( /[!"#$%&'()*+,.\/:;<=>?@\[\\\]^`{|}~]/g, '\\$&' );
};




